@@ -8,7 +8,7 @@ test_that("we can subset sf objects", {
   expect_equal(s1[,1], s1) #data.frame(x = c("x", "y")))
 
   expect_equal(nrow(s1[1,]), 1)
-  expect_equal(st_bbox(s1[1,]), c(xmin=1,ymin=2,xmax=1,ymax=2))
+  expect_equal(c(st_bbox(s1[1,])), c(xmin=1,ymin=2,xmax=1,ymax=2))
 
   a = c("x", "y")
   g = st_sfc(pt1, pt2)
@@ -23,7 +23,7 @@ test_that("we can create points sf from data.frame", {
   expect_identical(class(meuse_sf), c("sf", "data.frame"))
 })
 
-test_that("drop_zm works", {
+test_that("st_zm works", {
   pt = st_point(1:2)
   ptz = st_point(1:3, "XYZ")
   ptm = st_point(1:3, "XYM")
@@ -32,16 +32,18 @@ test_that("drop_zm works", {
   plz = st_multilinestring(list(matrix(1:15,,3), matrix(1:15,,3)), "XYZ")
   plm = st_multilinestring(list(matrix(1:15,,3), matrix(1:15,,3)), "XYM")
   plzm = st_multilinestring(list(matrix(1:20,,4), matrix(1:20,,4)), "XYZM")
-  expect_identical(pt, st_drop_zm(ptz))
-  expect_identical(pt, st_drop_zm(ptm))
-  expect_identical(pt, st_drop_zm(ptzm))
-  expect_identical(pl, st_drop_zm(plz))
-  expect_identical(pl, st_drop_zm(plm))
-  expect_identical(pl, st_drop_zm(plzm))
-  expect_identical(st_sfc(pt,pl), st_drop_zm(st_sfc(ptz,plz)))
-  expect_identical(st_sfc(pt,pl), st_drop_zm(st_sfc(ptm,plm)))
-  expect_identical(st_sfc(pt,pl), st_drop_zm(st_sfc(ptzm,plzm)))
-  expect_identical(st_sf(a = 1:2, geom = st_sfc(pt,pl)), st_drop_zm(st_sf(a = 1:2, geom = st_sfc(ptzm,plzm))))
+  expect_identical(pt, st_zm(ptz))
+  expect_identical(pt, st_zm(ptm))
+  expect_identical(pt, st_zm(ptzm))
+  expect_identical(pl, st_zm(plz))
+  expect_identical(pl, st_zm(plm))
+  expect_identical(pl, st_zm(plzm))
+  expect_identical(st_sfc(pt,pl), st_zm(st_sfc(ptz,plz)))
+  expect_identical(st_sfc(pt,pl), st_zm(st_sfc(ptm,plm)))
+  expect_identical(st_sfc(pt,pl), st_zm(st_sfc(ptzm,plzm)))
+  expect_identical(st_sf(a = 1:2, geom = st_sfc(pt,pl)), st_zm(st_sf(a = 1:2, geom = st_sfc(ptzm,plzm))))
+  expect_identical(st_zm(pt, drop = FALSE, what = "Z"), st_point(c(1:2,0)))
+  expect_silent(st_zm(pl, drop = FALSE, what = "Z"))
 })
 
 test_that("rbind/cbind work", {
